
# q-expansion of newform 8050.2.a.c, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8050
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [0], [-1], [-5], [-2], [5], [5], [-1], [-2], [-4], [-4], [-5], [8], [6], [4], [4], [10], [-1], [2], [15], [-10], [-3], [-15], [2], [0], [16], [-11], [-2], [13], [-6], [0], [-1], [17], [8], [-24], [-8], [-17], [24], [2], [-17], [-14], [-18], [11], [-22], [-2], [13], [-10], [12], [2], [18], [-8], [7], [-5], [-18], [-18], [2], [2], [2], [22], [5], [26], [7], [-10], [10], [-20], [19], [-23], [-23], [28], [-14], [-18], [-28], [-4], [-15], [6], [-24], [10], [33], [19], [-23], [8], [-28], [29], [24], [-1], [41], [-5], [-18], [20], [-36], [-18], [-30], [-36], [20], [-6], [6], [-25], [-21], [-14], [-23], [16], [-20], [-47], [-12], [3], [-7], [-33], [-8], [5], [-32], [-6], [18], [20], [20], [-14], [-16], [-48], [24], [49], [-30], [14], [36], [41], [21], [0], [-36], [-24], [6], [4], [12], [20], [-42], [-2], [19], [-41], [-20], [-28], [-34], [-26], [-28], [16], [-18], [39], [36], [-14], [-42], [-19], [25], [20], [28], [-14], [37], [-26], [44], [24], [-38], [6], [43], [0], [12], [-45], [38], [-3], [47], [32], [0], [-46], [37], [2], [-35], [30], [6], [-43], [-24], [-31], [-51], [-20], [10], [14], [-4], [5], [-14], [-29], [-54], [-12], [54], [19], [1], [22], [-17], [-9], [-23], [36], [-51], [-42], [31], [-38], [-9], [-54], [66], [58], [-22], [-27], [12], [-36], [-30], [8], [30], [8], [-2], [-60], [8], [-25], [-10], [55], [-36], [27], [-40], [24], [-36], [-50], [-15], [48], [7], [72], [-29], [-42], [-8], [9], [62], [55], [-50], [11], [4], [16], [-1], [6], [-19], [-12], [-19], [-41], [-46], [46], [47], [-10], [70], [-28], [69], [62], [34], [-67], [-22], [-15], [24], [1], [-20], [-39], [28], [61], [-2], [-49], [-30], [52], [-15], [25], [-30], [-30], [49], [24], [36], [-5], [-5], [62], [46], [-44], [-47], [22], [-66], [21], [-66], [52], [36], [-46], [69], [-60], [-30], [30], [-80], [-50], [-42], [4], [57], [23], [-26], [-24], [74], [76], [-20], [-31], [49], [62], [74], [-39], [89], [-1], [12], [-4], [36], [-10], [14], [-16], [-7], [-64], [-42], [-58], [65], [54], [-37], [-21], [56], [-13], [92], [84], [6], [-10], [25], [-20], [32], [0], [-66], [26], [0], [-73], [-89], [-36], [-2], [6], [-55], [-62], [28], [35], [72], [-40], [34], [-36], [28], [-37], [42], [-14], [47], [-58], [38], [-18], [4], [-61], [-62], [-67], [-42], [-66], [-8], [-7], [-58], [-96], [-7], [73], [48], [80], [5], [-36], [-15], [-62], [-62], [22], [-48], [65], [2], [43], [27], [22], [88], [-85], [8], [93], [-60], [-94], [-88], [12], [92], [42], [-5], [-24], [-79], [-59], [-62], [63], [80], [83], [-5], [-44], [58], [-57], [-82], [93], [48], [50], [104], [30], [12], [27], [99], [-60], [27], [88], [57], [-102], [-22], [-66], [33], [-78], [-48], [-62], [52], [-69], [89], [80], [-4], [15], [-91], [-36]]
